#####################################################################
#
#       Author : Kushal Das
#       Copyright (c)  2007 Kushal Das
#       kushal@fedoraproject.org
#
#####################################################################


import os
import translate.storage.po
import codecs

def dirwalk(dir):
    "walk a directory tree, using a generator"
    for f in os.listdir(dir):
        fullpath = os.path.join(dir,f)
        if os.path.isdir(fullpath) and not os.path.islink(fullpath):
            for x in dirwalk(fullpath):  # recurse into subdir
                yield x
        else:
            yield fullpath

def parsefile(f, text):
    """To parse a po file and get particular text"""
    po = translate.storage.po.pofile.parsefile(f)
    mesdict = []
    for u in po.units:
        try:
            if u.msgid[0].lower().find(text) != -1:
                mesdict.append(u)
        except:
            continue
    return mesdict

def findsingleword(f):
    """To parse a po file and get particular text"""
    po = translate.storage.po.pofile.parsefile(f)
    mesdict = []
    for u in po.units:
        try:
            if u.msgid[0].lower().find(' ') == -1:
                mesdict.append(u)
        except:
            continue
    return mesdict

def parsefile_wrongspellings(f, lang):
    """To parse a po file and get particular text"""
    po = translate.storage.po.pofile.parsefile(f)
    mesdict = []
    for u in po.units:
        try:
            if lang == 'all':
                if check_vowel(u.msgstr[0]):
                    mesdict.append(u)
            #if lang == 'bn':
            #    if check_bn(u.msgstr[0]):
            #        mesdict.append(u)
            #elif lang == "dev":
            #    if check_dev(u.msgstr[0]):
            #        mesdict.append(u)
            #elif lang == "gu":
            #    if check_gu(u.msgstr[0]):
            #        mesdict.append(u)
            #elif lang == "kn":
            #    if check_kn(u.msgstr[0]):
            #        mesdict.append(u)
            #elif lang == "ta":
            #    if check_ta(u.msgstr[0]):
            #        mesdict.append(u)
            #elif lang == "te":
            #    if check_te(u.msgstr[0]):
            #        mesdict.append(u)
            #elif lang == "or":
            #    if check_or(u.msgstr[0]):
            #        mesdict.append(u)
            #elif lang == "pa":
            #    if check_pa(u.msgstr[0]):
            #        mesdict.append(u)
        except:
            continue
    return mesdict

def getpofile(path):
    """To get all .po file names"""
    pofiles = []
    for x in dirwalk(path):
        if x.endswith(".po"):
            pofiles.append(x)
    return pofiles


def findtext(files, text):
    """Find the text in the list of files"""
    details = []
    for f in files:
        x = parsefile(f,text)
        if x:
            details.append((f,x))
    return details

def findwrongspellings(files, lang):
    """Find the text in the list of files"""
    details = []
    for f in files:
        x = parsefile_wrongspellings(f,lang)
        if x:
            details.append((f,x))
    return details

def check_vowel(text):
    bn = [u'\u09be',u'\u09bf',u'\u09c0',u'\u09c1',u'\u09c2',u'\u09c3',u'\u09c4',
            u'\u09c7',u'\u09c8',u'\u09cb',u'\u09cc',u'\u09cd',u'\u0981',u'\u0982',u'\u0983',u'\u0a3e',u'\u0a3f',u'\u0a40',u'\u0a41',u'\u0a42',u'\u0a47',u'\u0a48',u'\u0a4b',u'\u0a4c',u'\u0a4d',u'\u0a51',u'\u0a70',u'\u0a71',u'\u0abe',u'\u0abf',u'\u0ac0',u'\u0ac1',u'\u0ac2',u'\u0ac3',u'\u0ac4',u'\u0ac5',u'\u0ac7',u'\u0ac8',u'\u0ac9',u'\u0acb',u'\u0acc',u'\u0acd',u'\u0cbe',u'\u0cbf',u'\u0cc0',u'\u0cc1',u'\u0cc2',u'\u0cc3',u'\u0cc4',u'\u0cc6',u'\u0cc7',u'\u0cc8',u'\u0cc9',u'\u0cca',u'\u0ccb',u'\u0ccc',u'\u0ccd',u'\u0b3e',u'\u0b3f',u'\u0b40',u'\u0b41',u'\u0b42',u'\u0b43',u'\u0b44',u'\u0b47',u'\u0b48',u'\u0b4b',u'\u0b4c',u'\u0b4d',u'\u0b62',u'\u0b63',u'\u0bbe',u'\u0bbf',u'\u0bc0',u'\u0bc1',u'\u0bc2',u'\u0bc6',u'\u0bc7',u'\u0bc8',u'\u0bca',u'\u0bcb',u'\u0bcc',u'\u0bcd',u'\u0bd7',u'\u0c3e',u'\u0c3f',u'\u0c40',u'\u0c41',u'\u0c42',u'\u0c43',u'\u0c44',u'\u0c46',u'\u0c47',u'\u0c48',u'\u0c4a',u'\u0c4b',u'\u0c4c',u'\u0c4d',u'\u0c55',u'\u09ce',u'\u09cf',u'\u0940',u'\u0941',u'\u0942',u'\u0943',u'\u0944',u'\u0945',u'\u0946',u'\u0947',u'\u0948',u'\u0949',u'\u094a',u'\u094b',u'\u094c',u'\u094d' ]
    bn_exp1 = [u'\u09be',u'\u09bf',u'\u09c0',u'\u09c1',u'\u09c2',u'\u09c3',u'\u09c4',
            u'\u09c7',u'\u09c8',u'\u09cb',u'\u09cc']
    bn_exp2 = [u'\u0981',u'\u0982',u'\u0983']
    for pos, x in enumerate(text[:-1]):
        if x in bn:
            if text[pos+1] in bn:
                if x in bn_exp1 and text[pos+1] in bn_exp2:
                    continue
                else:
                    return True
    return False

def check_bn(text):
    bn = [u'\u09be',u'\u09bf',u'\u09c0',u'\u09c1',u'\u09c2',u'\u09c3',u'\u09c4',
            u'\u09c7',u'\u09c8',u'\u09cb',u'\u09cc',u'\u09cd',u'\u0981',u'\u0982',u'\u0983']
    bn_exp1 = [u'\u09be',u'\u09bf',u'\u09c0',u'\u09c1',u'\u09c2',u'\u09c3',u'\u09c4',
            u'09c7',u'\u09c8',u'\u09cb',u'\u09cc']
    bn_exp2 = [u'\u0981',u'\u0982',u'\u0983']
    for pos, x in enumerate(text[:-1]):
        if x in bn:
            if text[pos+1] in bn:
                if x in bn_exp1 and text[pos+1] in bn_exp2:
                    continue
                else:
                    return True
    return False

def check_pa(text):
    pa = [u'\u0a3e',u'\u0a3f',u'\u0a40',u'\u0a41',u'\u0a42',u'\u0a47',u'\u0a48',
          u'\u0a4b',u'\u0a4c',u'\u0a4d',u'\u0a51',u'\u0a70',u'\u0a71']
    for pos, x in enumerate(text[:-1]):
        if x in pa:
            if text[pos+1] in pa:
                return True
    return False

def check_gu(text):
    gu = [u'\u0abe',u'\u0abf',u'\u0ac0',u'\u0ac1',u'\u0ac2',u'\u0ac3',u'\u0ac4',
          u'\u0ac5',u'\u0ac7',u'\u0ac8',u'\u0ac9',u'\u0acb',u'\u0acc',u'\u0acd']
    for pos, x in enumerate(text[:-1]):
        if x in gu:
            if text[pos+1] in gu:
                return True
    return False

def check_kn(text):
    kn = [u'\u0cbe',u'\u0cbf',u'\u0cc0',u'\u0cc1',u'\u0cc2',u'\u0cc3',u'\u0cc4',
          u'\u0cc6',u'\u0cc7',u'\u0cc8',u'\u0cc9',u'\u0cca',u'\u0ccb',u'\u0ccc',u'\u0ccd']
    for pos, x in enumerate(text[:-1]):
        if x in kn:
            if text[pos+1] in kn:
                return True
    return False

def check_or(text):
    or_ = [u'\u0b3e',u'\u0b3f',u'\u0b40',u'\u0b41',u'\u0b42',u'\u0b43',u'\u0b44',
           u'\u0b47',u'\u0b48',u'\u0b4b',u'\u0b4c',u'\u0b4d',u'\u0b62',u'\u0b63']
    for pos, x in enumerate(text[:-1]):
        if x in or_:
            if text[pos+1] in or_:
                return True
    return False

def check_ta(text):
    ta = [u'\u0bbe',u'\u0bbf',u'\u0bc0',u'\u0bc1',u'\u0bc2',u'\u0bc6',u'\u0bc7',
            u'\u0bc8',u'\u0bca',u'\u0bcb',u'\u0bcc',u'\u0bcd',u'\u0bd7']
    for pos, x in enumerate(text[:-1]):
        if x in ta:
            if text[pos+1] in ta:
                return True
    return False

def check_te(text):
    te = [u'\u0c3e',u'\u0c3f',u'\u0c40',u'\u0c41',u'\u0c42',u'\u0c43',u'\u0c44',
          u'\u0c46',u'\u0c47',u'\u0c48',u'\u0c4a',u'\u0c4b',u'\u0c4c',u'\u0c4d',u'\u0c55']
    for pos, x in enumerate(text[:-1]):
        if x in te:
            if text[pos+1] in te:
                return True
    return False

def check_dev(text):
    dev = [u'\u09ce',u'\u09cf',u'\u0940',u'\u0941',u'\u0942',u'\u0943',u'\u0944',u'\u0945',
           u'\u0946',u'\u0947',u'\u0948',u'\u0949',u'\u094a',u'\u094b',u'\u094c',u'\u094d']
    for pos, x in enumerate(text[:-1]):
        if x in dev:
            if text[pos+1] in dev:
                return True
    return False


def findline(f,text):
    lines = [x.strip() for x in codecs.open(f, encoding="UTF-8").readlines()]
    for number, line in enumerate(lines):
        if line.find(text) != -1:
            return number + 1
    return -1
